"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryTransformBuiltinGlobalCall = void 0;
const lualib_1 = require("../utils/lualib");
const typescript_1 = require("../utils/typescript");
const call_1 = require("../visitors/call");
function tryTransformBuiltinGlobalCall(context, node, expressionType) {
    function getParameters() {
        const signature = context.checker.getResolvedSignature(node);
        return (0, call_1.transformArguments)(context, node.arguments, signature);
    }
    const name = expressionType.symbol.name;
    switch (name) {
        case "SymbolConstructor":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.Symbol, node, ...getParameters());
        case "NumberConstructor":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.Number, node, ...getParameters());
        case "isNaN":
        case "isFinite":
            const numberParameters = (0, typescript_1.isNumberType)(context, expressionType)
                ? getParameters()
                : [(0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.Number, undefined, ...getParameters())];
            return (0, lualib_1.transformLuaLibFunction)(context, name === "isNaN" ? lualib_1.LuaLibFeature.NumberIsNaN : lualib_1.LuaLibFeature.NumberIsFinite, node, ...numberParameters);
        case "parseFloat":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.ParseFloat, node, ...getParameters());
        case "parseInt":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.ParseInt, node, ...getParameters());
    }
}
exports.tryTransformBuiltinGlobalCall = tryTransformBuiltinGlobalCall;
//# sourceMappingURL=global.js.map