"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryTransformObjectPrototypeCall = exports.transformObjectConstructorCall = void 0;
const lua = require("../../LuaAST");
const diagnostics_1 = require("../utils/diagnostics");
const lualib_1 = require("../utils/lualib");
const call_1 = require("../visitors/call");
function transformObjectConstructorCall(context, node, calledMethod) {
    const args = (0, call_1.transformArguments)(context, node.arguments);
    const methodName = calledMethod.name.text;
    switch (methodName) {
        case "assign":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.ObjectAssign, node, ...args);
        case "defineProperty":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.ObjectDefineProperty, node, ...args);
        case "entries":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.ObjectEntries, node, ...args);
        case "fromEntries":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.ObjectFromEntries, node, ...args);
        case "getOwnPropertyDescriptor":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.ObjectGetOwnPropertyDescriptor, node, ...args);
        case "getOwnPropertyDescriptors":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.ObjectGetOwnPropertyDescriptors, node, ...args);
        case "groupBy":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.ObjectGroupBy, node, ...args);
        case "keys":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.ObjectKeys, node, ...args);
        case "values":
            return (0, lualib_1.transformLuaLibFunction)(context, lualib_1.LuaLibFeature.ObjectValues, node, ...args);
        default:
            context.diagnostics.push((0, diagnostics_1.unsupportedProperty)(calledMethod.name, "Object", methodName));
    }
}
exports.transformObjectConstructorCall = transformObjectConstructorCall;
function tryTransformObjectPrototypeCall(context, node, calledMethod) {
    const name = calledMethod.name.text;
    switch (name) {
        case "toString":
            const toStringIdentifier = lua.createIdentifier("tostring");
            return lua.createCallExpression(toStringIdentifier, [context.transformExpression(calledMethod.expression)], node);
        case "hasOwnProperty":
            const expr = context.transformExpression(calledMethod.expression);
            const signature = context.checker.getResolvedSignature(node);
            const parameters = (0, call_1.transformArguments)(context, node.arguments, signature);
            const rawGetIdentifier = lua.createIdentifier("rawget");
            const rawGetCall = lua.createCallExpression(rawGetIdentifier, [expr, ...parameters]);
            return lua.createBinaryExpression(rawGetCall, lua.createNilLiteral(), lua.SyntaxKind.InequalityOperator, node);
    }
}
exports.tryTransformObjectPrototypeCall = tryTransformObjectPrototypeCall;
//# sourceMappingURL=object.js.map