"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformForOfPairsKeyIterableStatement = exports.transformForOfPairsIterableStatement = exports.transformForOfIterableStatement = void 0;
const ts = require("typescript");
const lua = require("../../../LuaAST");
const utils_1 = require("../loops/utils");
const variable_declaration_1 = require("../variable-declaration");
const diagnostics_1 = require("../../utils/diagnostics");
const utils_2 = require("../../../utils");
const multi_1 = require("./multi");
function transformForOfMultiIterableStatement(context, statement, block, luaIterator, invalidMultiUseDiagnostic) {
    context.pushPrecedingStatements();
    let identifiers = [];
    if (ts.isVariableDeclarationList(statement.initializer)) {
        // Variables declared in for loop
        // for ${initializer} in ${iterable} do
        const binding = (0, utils_1.getVariableDeclarationBinding)(context, statement.initializer);
        if (ts.isArrayBindingPattern(binding)) {
            identifiers = binding.elements.map(e => (0, variable_declaration_1.transformArrayBindingElement)(context, e));
        }
        else {
            context.diagnostics.push(invalidMultiUseDiagnostic(binding));
        }
    }
    else if (ts.isArrayLiteralExpression(statement.initializer)) {
        // Variables NOT declared in for loop - catch iterator values in temps and assign
        // for ____value0 in ${iterable} do
        //     ${initializer} = ____value0
        identifiers = statement.initializer.elements.map((_, i) => lua.createIdentifier(`____value${i}`));
        if (identifiers.length > 0) {
            block.statements.unshift(lua.createAssignmentStatement(statement.initializer.elements.map(e => (0, utils_2.cast)(context.transformExpression(e), lua.isAssignmentLeftHandSideExpression)), identifiers));
        }
    }
    else {
        context.diagnostics.push(invalidMultiUseDiagnostic(statement.initializer));
    }
    if (identifiers.length === 0) {
        identifiers.push(lua.createAnonymousIdentifier());
    }
    block.statements.unshift(...context.popPrecedingStatements());
    return lua.createForInStatement(block, identifiers, [luaIterator], statement);
}
function transformForOfIterableStatement(context, statement, block) {
    var _a;
    const type = context.checker.getTypeAtLocation(statement.expression);
    if (((_a = type.aliasTypeArguments) === null || _a === void 0 ? void 0 : _a.length) === 2 && (0, multi_1.isMultiReturnType)(type.aliasTypeArguments[0])) {
        const luaIterator = context.transformExpression(statement.expression);
        return transformForOfMultiIterableStatement(context, statement, block, luaIterator, diagnostics_1.invalidMultiIterableWithoutDestructuring);
    }
    const luaIterator = context.transformExpression(statement.expression);
    const identifier = (0, utils_1.transformForInitializer)(context, statement.initializer, block);
    return lua.createForInStatement(block, [identifier], [luaIterator], statement);
}
exports.transformForOfIterableStatement = transformForOfIterableStatement;
function transformForOfPairsIterableStatement(context, statement, block) {
    const pairsCall = lua.createCallExpression(lua.createIdentifier("pairs"), [
        context.transformExpression(statement.expression),
    ]);
    return transformForOfMultiIterableStatement(context, statement, block, pairsCall, diagnostics_1.invalidPairsIterableWithoutDestructuring);
}
exports.transformForOfPairsIterableStatement = transformForOfPairsIterableStatement;
function transformForOfPairsKeyIterableStatement(context, statement, block) {
    const pairsCall = lua.createCallExpression(lua.createIdentifier("pairs"), [
        context.transformExpression(statement.expression),
    ]);
    const identifier = (0, utils_1.transformForInitializer)(context, statement.initializer, block);
    return lua.createForInStatement(block, [identifier], [pairsCall], statement);
}
exports.transformForOfPairsKeyIterableStatement = transformForOfPairsKeyIterableStatement;
//# sourceMappingURL=iterable.js.map