"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPlugins = void 0;
const utils_1 = require("./utils");
const performance = require("../measure-performance");
function getPlugins(program) {
    var _a;
    performance.startSection("getPlugins");
    const diagnostics = [];
    const pluginsFromOptions = [];
    const options = program.getCompilerOptions();
    for (const [index, pluginOption] of ((_a = options.luaPlugins) !== null && _a !== void 0 ? _a : []).entries()) {
        const optionName = `tstl.luaPlugins[${index}]`;
        const { error: resolveError, result: factory } = (0, utils_1.resolvePlugin)("plugin", `${optionName}.name`, (0, utils_1.getConfigDirectory)(options), pluginOption.name, pluginOption.import);
        if (resolveError)
            diagnostics.push(resolveError);
        if (factory === undefined)
            continue;
        const plugin = typeof factory === "function" ? factory(pluginOption) : factory;
        pluginsFromOptions.push(plugin);
    }
    if (options.tstlVerbose) {
        console.log(`Loaded ${pluginsFromOptions.length} plugins`);
    }
    performance.endSection("getPlugins");
    return { diagnostics, plugins: pluginsFromOptions };
}
exports.getPlugins = getPlugins;
//# sourceMappingURL=plugins.js.map